@@ -16,14 +16,14 @@
 import logging
 import sys
 
-from osc_lib.cli import parseractions
 from osc_lib.command import command
 from osc_lib import exceptions as exc
 from osc_lib import utils
 from oslo_serialization import jsonutils
-import six
-from six.moves.urllib import request
+from urllib import request
+import yaml
 
+from heatclient._i18n import _
 from heatclient.common import event_utils
 from heatclient.common import format_utils
 from heatclient.common import hook_utils
@@ -31,8 +31,6 @@ from heatclient.common import http
 from heatclient.common import template_utils
 from heatclient.common import utils as heat_utils
 from heatclient import exc as heat_exc
-from heatclient.openstack.common._i18n import _
-from heatclient.openstack.common._i18n import _LI
 
 
 class CreateStack(command.ShowOne):
@@ -48,6 +46,13 @@ class CreateStack(command.ShowOne):
             action='append',
             help=_('Path to the environment. Can be specified multiple times')
         )
+        parser.add_argument(
+            '-s', '--files-container',
+            metavar='<files-container>',
+            help=_('Swift files container name. Local files other than '
+                   'root template would be ignored. If other files are not '
+                   'found in swift, heat engine would raise an error.')
+        )
         parser.add_argument(
             '--timeout',
             metavar='<timeout>',
@@ -61,9 +66,9 @@ class CreateStack(command.ShowOne):
             action='append',
             help=_('Name of a resource to set a pre-create hook to. Resources '
                    'in nested stacks can be set using slash as a separator: '
-                   'nested_stack/another/my_resource. You can use wildcards '
-                   'to match multiple stacks or resources: '
-                   'nested_stack/an*/*_resource. This can be specified '
+                   '``nested_stack/another/my_resource``. You can use '
+                   'wildcards to match multiple stacks or resources: '
+                   '``nested_stack/an*/*_resource``. This can be specified '
                    'multiple times')
         )
         parser.add_argument(
@@ -91,6 +96,14 @@ class CreateStack(command.ShowOne):
             action='store_true',
             help=_('Wait until stack goes to CREATE_COMPLETE or CREATE_FAILED')
         )
+        parser.add_argument(
+            '--poll',
+            metavar='SECONDS',
+            type=int,
+            default=5,
+            help=_('Poll interval in seconds for use with --wait, '
+                   'defaults to 5.')
+        )
         parser.add_argument(
             '--tags',
             metavar='<tag1,tag2...>',
@@ -123,11 +136,15 @@ class CreateStack(command.ShowOne):
 
         tpl_files, template = template_utils.process_template_path(
             parsed_args.template,
-            object_request=http.authenticated_fetcher(client))
+            object_request=http.authenticated_fetcher(client),
+            fetch_child=parsed_args.files_container is None)
 
+        env_files_list = []
         env_files, env = (
             template_utils.process_multiple_environments_and_files(
-                env_paths=parsed_args.environment))
+                env_paths=parsed_args.environment,
+                env_list_tracker=env_files_list,
+                fetch_env_files=parsed_args.files_container is None))
 
         parameters = heat_utils.format_all_parameters(
             parsed_args.parameter,
@@ -147,6 +164,13 @@ class CreateStack(command.ShowOne):
             'environment': env
         }
 
+        # If one or more environments is found, pass the listing to the server
+        if env_files_list:
+            fields['environment_files'] = env_files_list
+
+        if parsed_args.files_container:
+            fields['files_container'] = parsed_args.files_container
+
         if parsed_args.tags:
             fields['tags'] = parsed_args.tags
         if parsed_args.timeout:
@@ -179,7 +203,8 @@ class CreateStack(command.ShowOne):
         stack = client.stacks.create(**fields)['stack']
         if parsed_args.wait:
             stack_status, msg = event_utils.poll_for_events(
-                client, parsed_args.name, action='CREATE')
+                client, parsed_args.name, action='CREATE',
+                poll_period=parsed_args.poll)
             if stack_status == 'CREATE_FAILED':
                 raise exc.CommandError(msg)
 
@@ -197,6 +222,13 @@ class UpdateStack(command.ShowOne):
             '-t', '--template', metavar='<template>',
             help=_('Path to the template')
         )
+        parser.add_argument(
+            '-s', '--files-container',
+            metavar='<files-container>',
+            help=_('Swift files container name. Local files other than '
+                   'root template would be ignored. If other files are not '
+                   'found in swift, heat engine would raise an error.')
+        )
         parser.add_argument(
             '-e', '--environment', metavar='<environment>',
             action='append',
@@ -206,9 +238,9 @@ class UpdateStack(command.ShowOne):
             '--pre-update', metavar='<resource>', action='append',
             help=_('Name of a resource to set a pre-update hook to. Resources '
                    'in nested stacks can be set using slash as a separator: '
-                   'nested_stack/another/my_resource. You can use wildcards '
-                   'to match multiple stacks or resources: '
-                   'nested_stack/an*/*_resource. This can be specified '
+                   '``nested_stack/another/my_resource``. You can use '
+                   'wildcards to match multiple stacks or resources: '
+                   '``nested_stack/an*/*_resource``. This can be specified '
                    'multiple times')
         )
         parser.add_argument(
@@ -228,6 +260,10 @@ class UpdateStack(command.ShowOne):
             help=_('Do not actually perform the stack update, but show what '
                    'would be changed')
         )
+        parser.add_argument(
+            '--show-nested', default=False, action="store_true",
+            help=_('Show nested stacks when performing --dry-run')
+        )
         parser.add_argument(
             '--parameter', metavar='<key=value>',
             help=_('Parameter values used to create the stack. '
@@ -274,6 +310,11 @@ class UpdateStack(command.ShowOne):
             help=_('Wait until stack goes to UPDATE_COMPLETE or '
                    'UPDATE_FAILED')
         )
+        parser.add_argument(
+            '--converge',
+            action='store_true',
+            help=_('Stack update with observe on reality.')
+        )
 
         return parser
 
@@ -285,11 +326,15 @@ class UpdateStack(command.ShowOne):
         tpl_files, template = template_utils.process_template_path(
             parsed_args.template,
             object_request=http.authenticated_fetcher(client),
-            existing=parsed_args.existing)
+            existing=parsed_args.existing,
+            fetch_child=parsed_args.files_container is None)
 
+        env_files_list = []
         env_files, env = (
             template_utils.process_multiple_environments_and_files(
-                env_paths=parsed_args.environment))
+                env_paths=parsed_args.environment,
+                env_list_tracker=env_files_list,
+                fetch_env_files=parsed_args.files_container is None))
 
         parameters = heat_utils.format_all_parameters(
             parsed_args.parameter,
@@ -309,6 +354,13 @@ class UpdateStack(command.ShowOne):
             'environment': env
         }
 
+        # If one or more environments is found, pass the listing to the server
+        if env_files_list:
+            fields['environment_files'] = env_files_list
+
+        if parsed_args.files_container:
+            fields['files_container'] = parsed_args.files_container
+
         if parsed_args.tags:
             fields['tags'] = parsed_args.tags
         if parsed_args.timeout:
@@ -325,6 +377,9 @@ class UpdateStack(command.ShowOne):
                 fields['disable_rollback'] = rollback == 'disabled'
 
         if parsed_args.dry_run:
+            if parsed_args.show_nested:
+                fields['show_nested'] = parsed_args.show_nested
+
             changes = client.stacks.preview_update(**fields)
 
             fields = ['state', 'resource_name', 'resource_type',
@@ -344,6 +399,9 @@ class UpdateStack(command.ShowOne):
                                             limit=1)
             marker = events[0].id if events else None
 
+        if parsed_args.converge:
+            fields['converge'] = True
+
         client.stacks.update(**fields)
 
         if parsed_args.wait:
@@ -369,19 +427,28 @@ class ShowStack(command.ShowOne):
             metavar='<stack>',
             help='Stack to display (name or ID)',
         )
+        parser.add_argument(
+            '--no-resolve-outputs', action="store_true",
+            help=_('Do not resolve outputs of the stack.')
+        )
         return parser
 
     def take_action(self, parsed_args):
         self.log.debug("take_action(%s)", parsed_args)
 
         heat_client = self.app.client_manager.orchestration
-        return _show_stack(heat_client, stack_id=parsed_args.stack,
-                           format=parsed_args.formatter)
+        return _show_stack(
+            heat_client, stack_id=parsed_args.stack,
+            format=parsed_args.formatter,
+            resolve_outputs=(not parsed_args.no_resolve_outputs))
 
 
-def _show_stack(heat_client, stack_id, format='', short=False):
+def _show_stack(heat_client, stack_id, format='', short=False,
+                resolve_outputs=True):
     try:
-        data = heat_client.stacks.get(stack_id=stack_id)
+        _resolve_outputs = not short and resolve_outputs
+        data = heat_client.stacks.get(stack_id=stack_id,
+                                      resolve_outputs=_resolve_outputs)
     except heat_exc.HTTPNotFound:
         raise exc.CommandError('Stack not found: %s' % stack_id)
     else:
@@ -397,11 +464,10 @@ def _show_stack(heat_client, stack_id, format='', short=False):
         ]
 
         if not short:
-            columns += [
-                'parameters',
-                'outputs',
-                'links',
-            ]
+            columns.append('parameters')
+            if _resolve_outputs:
+                columns.append('outputs')
+            columns.append('links')
 
             exclude_columns = ('template_description',)
             for key in data.to_dict():
@@ -453,7 +519,7 @@ class ListStack(command.Lister):
             metavar='<key=value>',
             help=_('Filter properties to apply on returned stacks (repeat to '
                    'filter on multiple properties)'),
-            action=parseractions.KeyValueAction
+            action='append'
         )
         parser.add_argument(
             '--tags',
@@ -471,6 +537,7 @@ class ListStack(command.Lister):
         parser.add_argument(
             '--limit',
             metavar='<limit>',
+            type=int,
             help=_('The number of stacks returned')
         )
         parser.add_argument(
@@ -612,8 +679,6 @@ def _list(client, args=None):
             columns.pop()
         if args.long:
             columns.insert(2, 'Stack Owner')
-        if args.long or args.all_projects:
-            columns.insert(2, 'Project')
 
         if args.nested:
             columns.append('Parent')
@@ -623,6 +688,11 @@ def _list(client, args=None):
             columns.append('Deletion Time')
 
     data = client.stacks.list(**kwargs)
+    data = list(data)
+    for stk in data:
+        if hasattr(stk, 'project'):
+            columns.insert(2, 'Project')
+            break
     data = utils.sort_items(data, args.sort if args else None)
 
     return (
@@ -645,7 +715,7 @@ class DeleteStack(command.Command):
             help=_('Stack(s) to delete (name or ID)')
         )
         parser.add_argument(
-            '--yes',
+            '-y', '--yes',
             action='store_true',
             help=_('Skip yes/no prompt (assume yes)')
         )
@@ -663,20 +733,20 @@ class DeleteStack(command.Command):
 
         try:
             if not parsed_args.yes and sys.stdin.isatty():
-                sys.stdout.write(
-                    _("Are you sure you want to delete this stack(s) [y/N]? "))
-                prompt_response = sys.stdin.readline().lower()
+                prompt_response = input(
+                    _("Are you sure you want to delete this stack(s) [y/N]? ")
+                ).lower()
                 if not prompt_response.startswith('y'):
-                    self.log.info(_LI('User did not confirm stack delete so '
-                                      'taking no action.'))
+                    self.log.info('User did not confirm stack delete so '
+                                  'taking no action.')
                     return
         except KeyboardInterrupt:  # ctrl-c
-            self.log.info(_LI('User did not confirm stack delete '
-                              '(ctrl-c) so taking no action.'))
+            self.log.info('User did not confirm stack delete '
+                          '(ctrl-c) so taking no action.')
             return
         except EOFError:  # ctrl-d
-            self.log.info(_LI('User did not confirm stack delete '
-                              '(ctrl-d) so taking no action.'))
+            self.log.info('User did not confirm stack delete '
+                          '(ctrl-d) so taking no action.')
             return
 
         failure_count = 0
@@ -786,13 +856,15 @@ class AdoptStack(command.ShowOne):
         adopt_url = heat_utils.normalise_file_path_to_url(
             parsed_args.adopt_file)
         adopt_data = request.urlopen(adopt_url).read().decode('utf-8')
-
+        yaml_adopt_data = yaml.safe_load(adopt_data) or {}
+        files = yaml_adopt_data.get('files', {})
+        files.update(env_files)
         fields = {
             'stack_name': parsed_args.name,
             'disable_rollback': not parsed_args.enable_rollback,
             'adopt_stack_data': adopt_data,
             'parameters': heat_utils.format_parameters(parsed_args.parameter),
-            'files': dict(list(env_files.items())),
+            'files': files,
             'environment': env,
             'timeout': parsed_args.timeout
         }
@@ -847,8 +919,52 @@ class AbandonStack(format_utils.JsonFormat):
             except IOError as e:
                 raise exc.CommandError(str(e))
 
-        data = list(six.itervalues(stack))
-        columns = list(six.iterkeys(stack))
+        data = list(stack.values())
+        columns = list(stack.keys())
+        return columns, data
+
+
+class ExportStack(format_utils.JsonFormat):
+    """Export stack data json."""
+
+    log = logging.getLogger(__name__ + '.ExportStack')
+
+    def get_parser(self, prog_name):
+        parser = super(ExportStack, self).get_parser(prog_name)
+        parser.add_argument(
+            'stack',
+            metavar='<stack>',
+            help=_('Name or ID of stack to export')
+        )
+        parser.add_argument(
+            '--output-file',
+            metavar='<output-file>',
+            help=_('File to output export data')
+        )
+
+        return parser
+
+    def take_action(self, parsed_args):
+        self.log.debug('take_action(%s)', parsed_args)
+
+        client = self.app.client_manager.orchestration
+
+        try:
+            data_info = client.stacks.export(stack_id=parsed_args.stack)
+        except heat_exc.HTTPNotFound:
+            msg = _('Stack not found: %s') % parsed_args.stack
+            raise exc.CommandError(msg)
+
+        if parsed_args.output_file is not None:
+            try:
+                with open(parsed_args.output_file, 'w') as f:
+                    f.write(jsonutils.dumps(data_info, indent=2))
+                    return [], None
+            except IOError as e:
+                raise exc.CommandError(str(e))
+
+        data = list(data_info.values())
+        columns = list(data_info.keys())
         return columns, data
 
 
@@ -929,11 +1045,6 @@ class OutputShowStack(command.ShowOne):
             msg = _('Output error: %s') % output['output_error']
             raise exc.CommandError(msg)
 
-        if (isinstance(output['output_value'], list) or
-                isinstance(output['output_value'], dict)):
-            output['output_value'] = heat_utils.json_formatter(
-                output['output_value'])
-
         return self.dict2columns(output)
 
 
@@ -1142,17 +1253,25 @@ class CancelStack(StackActionBase):
     """Cancel current task for a stack.
 
     Supported tasks for cancellation:
+
     * update
+    * create
     """
 
     log = logging.getLogger(__name__ + '.CancelStack')
 
     def get_parser(self, prog_name):
-        return self._get_parser(
+        parser = self._get_parser(
             prog_name,
             _('Stack(s) to cancel (name or ID)'),
-            _('Wait for check to complete')
+            _('Wait for cancel to complete')
         )
+        parser.add_argument(
+            '--no-rollback',
+            action='store_true',
+            help=_('Cancel without rollback')
+        )
+        return parser
 
     def take_action(self, parsed_args):
         self.log.debug("take_action(%s)", parsed_args)
@@ -1165,20 +1284,25 @@ class CancelStack(StackActionBase):
             'Updated Time'
         ]
         heat_client = self.app.client_manager.orchestration
-
+        if parsed_args.no_rollback:
+            action = heat_client.actions.cancel_without_rollback
+            allowed_statuses = ['create_in_progress',
+                                'update_in_progress']
+        else:
+            action = heat_client.actions.cancel_update
+            allowed_statuses = ['update_in_progress']
         for stack in parsed_args.stack:
             try:
                 data = heat_client.stacks.get(stack_id=stack)
             except heat_exc.HTTPNotFound:
                 raise exc.CommandError('Stack not found: %s' % stack)
-
             status = getattr(data, 'stack_status').lower()
-            if status == 'update_in_progress':
+            if status in allowed_statuses:
                 data = _stack_action(
                     stack,
                     parsed_args,
                     heat_client,
-                    heat_client.actions.cancel_update
+                    action
                 )
                 rows += [utils.get_dict_properties(data.to_dict(), columns)]
             else:
@@ -1292,9 +1416,9 @@ class StackHookClear(command.Command):
             nargs='+',
             help=_('Resource names with hooks to clear. Resources '
                    'in nested stacks can be set using slash as a separator: '
-                   'nested_stack/another/my_resource. You can use wildcards '
-                   'to match multiple stacks or resources: '
-                   'nested_stack/an*/*_resource')
+                   '``nested_stack/another/my_resource``. You can use '
+                   'wildcards to match multiple stacks or resources: '
+                   '``nested_stack/an*/*_resource``')
         )
         return parser
 
